"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createHorizontalStrength = createHorizontalStrength;
exports.createVerticalStrength = createVerticalStrength;
exports["default"] = createScrollingComponent;
exports.defaultVerticalStrength = exports.defaultHorizontalStrength = void 0;
exports.useDndScrolling = useDndScrolling;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _defaults = _interopRequireDefault(require("defaults"));

var _reactDnd = require("react-dnd");

var _lodash = _interopRequireDefault(require("lodash.throttle"));

var _raf = _interopRequireDefault(require("raf"));

var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));

var _util = require("./util.js");

var _excluded = ["strengthMultiplier", "verticalStrength", "horizontalStrength", "onScrollChange", "dragDropManager"];

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var DEFAULT_BUFFER = 150;

var getDisplayName = function getDisplayName(PassedComponent) {
  return PassedComponent.displayName || PassedComponent.name || (typeof PassedComponent === 'string' && PassedComponent.length > 0 ? PassedComponent : 'Unknown');
};

function createHorizontalStrength(_buffer) {
  return function defaultHorizontalStrength(_ref, point) {
    var x = _ref.x,
        w = _ref.w,
        y = _ref.y,
        h = _ref.h;
    var buffer = Math.min(w / 2, _buffer);
    var inRange = point.x >= x && point.x <= x + w;
    var inBox = inRange && point.y >= y && point.y <= y + h;

    if (inBox) {
      if (point.x < x + buffer) {
        return (point.x - x - buffer) / buffer;
      } else if (point.x > x + w - buffer) {
        return -(x + w - point.x - buffer) / buffer;
      }
    }

    return 0;
  };
}

function createVerticalStrength(_buffer) {
  return function defaultVerticalStrength(_ref2, point) {
    var y = _ref2.y,
        h = _ref2.h,
        x = _ref2.x,
        w = _ref2.w;
    var buffer = Math.min(h / 2, _buffer);
    var inRange = point.y >= y && point.y <= y + h;
    var inBox = inRange && point.x >= x && point.x <= x + w;

    if (inBox) {
      if (point.y < y + buffer) {
        return (point.y - y - buffer) / buffer;
      } else if (point.y > y + h - buffer) {
        return -(y + h - point.y - buffer) / buffer;
      }
    }

    return 0;
  };
}

var defaultHorizontalStrength = createHorizontalStrength(DEFAULT_BUFFER);
exports.defaultHorizontalStrength = defaultHorizontalStrength;
var defaultVerticalStrength = createVerticalStrength(DEFAULT_BUFFER);
exports.defaultVerticalStrength = defaultVerticalStrength;
var defaultProps = {
  onScrollChange: _util.noop,
  verticalStrength: defaultVerticalStrength,
  horizontalStrength: defaultHorizontalStrength,
  strengthMultiplier: 30,
  dragDropManager: null
};

function useDndScrolling(componentRef, passedOptions) {
  var _props$dragDropManage;

  var props = (0, _defaults["default"])(passedOptions, defaultProps);
  var scaleX = (0, _react.useRef)(0);
  var scaleY = (0, _react.useRef)(0);
  var frame = (0, _react.useRef)(0);
  var attached = (0, _react.useRef)(false);
  var dragging = (0, _react.useRef)(false);

  var _useContext = (0, _react.useContext)(_reactDnd.DndContext),
      dragDropManagerCtx = _useContext.dragDropManager;

  var dragDropManager = (_props$dragDropManage = props.dragDropManager) !== null && _props$dragDropManage !== void 0 ? _props$dragDropManage : dragDropManagerCtx;

  if (!dragDropManager) {
    throw new Error('Unable to get dragDropManager from context. Provide DnDContext or pass dragDropManager via options to useDndScrolling.');
  }

  var monitor = _react["default"].useMemo(function () {
    return dragDropManager.getMonitor();
  }, [dragDropManager]);

  var startScrolling = _react["default"].useCallback(function () {
    var i = 0;

    var tick = function tick() {
      var strengthMultiplier = props.strengthMultiplier,
          onScrollChange = props.onScrollChange; // stop scrolling if there's nothing to do

      if (strengthMultiplier === 0 || scaleX.current + scaleY.current === 0) {
        // eslint-disable-next-line no-use-before-define
        stopScrolling();
        return;
      } // there's a bug in safari where it seems like we can't get
      // mousemove events from a container that also emits a scroll
      // event that same frame. So we double the strengthMultiplier and only adjust
      // the scroll position at 30fps


      if (i++ % 2) {
        var _componentRef$current = componentRef.current,
            scrollLeft = _componentRef$current.scrollLeft,
            scrollTop = _componentRef$current.scrollTop,
            scrollWidth = _componentRef$current.scrollWidth,
            scrollHeight = _componentRef$current.scrollHeight,
            clientWidth = _componentRef$current.clientWidth,
            clientHeight = _componentRef$current.clientHeight;
        var newLeft = scaleX.current ? // eslint-disable-next-line no-param-reassign
        componentRef.current.scrollLeft = (0, _util.intBetween)(0, scrollWidth - clientWidth, scrollLeft + scaleX.current * strengthMultiplier) : scrollLeft;
        var newTop = scaleY.current ? // eslint-disable-next-line no-param-reassign
        componentRef.current.scrollTop = (0, _util.intBetween)(0, scrollHeight - clientHeight, scrollTop + scaleY.current * strengthMultiplier) : scrollTop;
        onScrollChange(newLeft, newTop);
      }

      frame.current = (0, _raf["default"])(tick);
    };

    tick();
  }, []); // Update scaleX and scaleY every 100ms or so
  // and start scrolling if necessary


  var updateScrolling = _react["default"].useMemo(function () {
    return (0, _lodash["default"])(function (evt) {
      if (!componentRef.current) {
        return;
      }

      var _componentRef$current2 = componentRef.current.getBoundingClientRect(),
          x = _componentRef$current2.left,
          y = _componentRef$current2.top,
          w = _componentRef$current2.width,
          h = _componentRef$current2.height;

      var box = {
        x: x,
        y: y,
        w: w,
        h: h
      };
      var coords = (0, _util.getCoords)(evt); // calculate strength

      scaleX.current = props.horizontalStrength(box, coords);
      scaleY.current = props.verticalStrength(box, coords); // start scrolling if we need to

      if (!frame.current && (scaleX.current || scaleY.current)) {
        startScrolling();
      }
    }, 100, {
      trailing: false
    });
  }, []);

  var attach = (0, _react.useCallback)(function () {
    attached.current = true;
    window.document.body.addEventListener('dragover', updateScrolling);
    window.document.body.addEventListener('touchmove', updateScrolling);
  }, []);
  var detach = (0, _react.useCallback)(function () {
    attached.current = false;
    window.document.body.removeEventListener('dragover', updateScrolling);
    window.document.body.removeEventListener('touchmove', updateScrolling);
  }, []);

  var stopScrolling = _react["default"].useCallback(function () {
    detach();
    scaleX.current = 0;
    scaleY.current = 0;

    if (frame.current) {
      _raf["default"].cancel(frame.current);

      frame.current = null;
    }
  }, []);

  var handleEvent = (0, _react.useCallback)(function (evt) {
    if (dragging.current && !attached.current) {
      attach();
      updateScrolling(evt);
    }
  }, [dragDropManager]);
  (0, _react.useEffect)(function () {
    if (!dragging.current && monitor.isDragging()) {
      dragging.current = true;
    } else if (dragging.current && !monitor.isDragging()) {
      dragging.current = false;
      stopScrolling();
    }
  }, [monitor.isDragging()]);
  (0, _react.useEffect)(function () {
    if (!componentRef.current) {
      return function () {};
    }

    componentRef.current.addEventListener('dragover', handleEvent); // touchmove events don't seem to work across siblings, so we unfortunately
    // have to attach the listeners to the body

    window.document.body.addEventListener('touchmove', handleEvent);
    return function () {
      if (componentRef.current) {
        componentRef.current.removeEventListener('dragover', handleEvent);
      }

      window.document.body.removeEventListener('touchmove', handleEvent);
      stopScrolling();
    };
  }, [dragDropManager]);
}

function createScrollingComponent(WrappedComponent) {
  function ScrollingComponent(_ref3) {
    var strengthMultiplier = _ref3.strengthMultiplier,
        verticalStrength = _ref3.verticalStrength,
        horizontalStrength = _ref3.horizontalStrength,
        onScrollChange = _ref3.onScrollChange,
        dragDropManager = _ref3.dragDropManager,
        passedProps = _objectWithoutProperties(_ref3, _excluded);

    var ref = (0, _react.useRef)(null);
    useDndScrolling(ref, {
      strengthMultiplier: strengthMultiplier,
      verticalStrength: verticalStrength,
      horizontalStrength: horizontalStrength,
      onScrollChange: onScrollChange,
      dragDropManager: dragDropManager
    });
    return /*#__PURE__*/_react["default"].createElement(WrappedComponent, _extends({}, passedProps, {
      ref: ref
    }));
  }

  ScrollingComponent.displayName = "Scrolling(".concat(getDisplayName(WrappedComponent), ")");
  ScrollingComponent.propTypes = {
    onScrollChange: _propTypes["default"].func,
    verticalStrength: _propTypes["default"].func,
    horizontalStrength: _propTypes["default"].func,
    strengthMultiplier: _propTypes["default"].number,
    dragDropManager: _propTypes["default"].any
  };
  ScrollingComponent.defaultProps = defaultProps;
  return (0, _hoistNonReactStatics["default"])(ScrollingComponent, WrappedComponent);
}