var _excluded = ["strengthMultiplier", "verticalStrength", "horizontalStrength", "onScrollChange", "dragDropManager"];

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { useEffect, useRef, useCallback, useContext } from 'react';
import PropTypes from 'prop-types';
import defaults from 'defaults';
import { DndContext } from 'react-dnd';
import throttle from 'lodash.throttle';
import raf from 'raf';
import hoist from 'hoist-non-react-statics';
import { noop, intBetween, getCoords } from './util.js';
import { jsx as _jsx } from "react/jsx-runtime";
var DEFAULT_BUFFER = 150;

var getDisplayName = function getDisplayName(PassedComponent) {
  return PassedComponent.displayName || PassedComponent.name || (typeof PassedComponent === 'string' && PassedComponent.length > 0 ? PassedComponent : 'Unknown');
};

export function createHorizontalStrength(_buffer) {
  return function defaultHorizontalStrength(_ref, point) {
    var x = _ref.x,
        w = _ref.w,
        y = _ref.y,
        h = _ref.h;
    var buffer = Math.min(w / 2, _buffer);
    var inRange = point.x >= x && point.x <= x + w;
    var inBox = inRange && point.y >= y && point.y <= y + h;

    if (inBox) {
      if (point.x < x + buffer) {
        return (point.x - x - buffer) / buffer;
      } else if (point.x > x + w - buffer) {
        return -(x + w - point.x - buffer) / buffer;
      }
    }

    return 0;
  };
}
export function createVerticalStrength(_buffer) {
  return function defaultVerticalStrength(_ref2, point) {
    var y = _ref2.y,
        h = _ref2.h,
        x = _ref2.x,
        w = _ref2.w;
    var buffer = Math.min(h / 2, _buffer);
    var inRange = point.y >= y && point.y <= y + h;
    var inBox = inRange && point.x >= x && point.x <= x + w;

    if (inBox) {
      if (point.y < y + buffer) {
        return (point.y - y - buffer) / buffer;
      } else if (point.y > y + h - buffer) {
        return -(y + h - point.y - buffer) / buffer;
      }
    }

    return 0;
  };
}
export var defaultHorizontalStrength = createHorizontalStrength(DEFAULT_BUFFER);
export var defaultVerticalStrength = createVerticalStrength(DEFAULT_BUFFER);
var defaultProps = {
  onScrollChange: noop,
  verticalStrength: defaultVerticalStrength,
  horizontalStrength: defaultHorizontalStrength,
  strengthMultiplier: 30,
  dragDropManager: null
};
export function useDndScrolling(componentRef, passedOptions) {
  var _props$dragDropManage;

  var props = defaults(passedOptions, defaultProps);
  var scaleX = useRef(0);
  var scaleY = useRef(0);
  var frame = useRef(0);
  var attached = useRef(false);
  var dragging = useRef(false);

  var _useContext = useContext(DndContext),
      dragDropManagerCtx = _useContext.dragDropManager;

  var dragDropManager = (_props$dragDropManage = props.dragDropManager) != null ? _props$dragDropManage : dragDropManagerCtx;

  if (!dragDropManager) {
    throw new Error('Unable to get dragDropManager from context. Provide DnDContext or pass dragDropManager via options to useDndScrolling.');
  }

  var monitor = React.useMemo(function () {
    return dragDropManager.getMonitor();
  }, [dragDropManager]);
  var startScrolling = React.useCallback(function () {
    var i = 0;

    var tick = function tick() {
      var strengthMultiplier = props.strengthMultiplier,
          onScrollChange = props.onScrollChange; // stop scrolling if there's nothing to do

      if (strengthMultiplier === 0 || scaleX.current + scaleY.current === 0) {
        // eslint-disable-next-line no-use-before-define
        stopScrolling();
        return;
      } // there's a bug in safari where it seems like we can't get
      // mousemove events from a container that also emits a scroll
      // event that same frame. So we double the strengthMultiplier and only adjust
      // the scroll position at 30fps


      if (i++ % 2) {
        var _componentRef$current = componentRef.current,
            scrollLeft = _componentRef$current.scrollLeft,
            scrollTop = _componentRef$current.scrollTop,
            scrollWidth = _componentRef$current.scrollWidth,
            scrollHeight = _componentRef$current.scrollHeight,
            clientWidth = _componentRef$current.clientWidth,
            clientHeight = _componentRef$current.clientHeight;
        var newLeft = scaleX.current ? // eslint-disable-next-line no-param-reassign
        componentRef.current.scrollLeft = intBetween(0, scrollWidth - clientWidth, scrollLeft + scaleX.current * strengthMultiplier) : scrollLeft;
        var newTop = scaleY.current ? // eslint-disable-next-line no-param-reassign
        componentRef.current.scrollTop = intBetween(0, scrollHeight - clientHeight, scrollTop + scaleY.current * strengthMultiplier) : scrollTop;
        onScrollChange(newLeft, newTop);
      }

      frame.current = raf(tick);
    };

    tick();
  }, []); // Update scaleX and scaleY every 100ms or so
  // and start scrolling if necessary

  var updateScrolling = React.useMemo(function () {
    return throttle(function (evt) {
      if (!componentRef.current) {
        return;
      }

      var _componentRef$current2 = componentRef.current.getBoundingClientRect(),
          x = _componentRef$current2.left,
          y = _componentRef$current2.top,
          w = _componentRef$current2.width,
          h = _componentRef$current2.height;

      var box = {
        x: x,
        y: y,
        w: w,
        h: h
      };
      var coords = getCoords(evt); // calculate strength

      scaleX.current = props.horizontalStrength(box, coords);
      scaleY.current = props.verticalStrength(box, coords); // start scrolling if we need to

      if (!frame.current && (scaleX.current || scaleY.current)) {
        startScrolling();
      }
    }, 100, {
      trailing: false
    });
  }, []);
  var attach = useCallback(function () {
    attached.current = true;
    window.document.body.addEventListener('dragover', updateScrolling);
    window.document.body.addEventListener('touchmove', updateScrolling);
  }, []);
  var detach = useCallback(function () {
    attached.current = false;
    window.document.body.removeEventListener('dragover', updateScrolling);
    window.document.body.removeEventListener('touchmove', updateScrolling);
  }, []);
  var stopScrolling = React.useCallback(function () {
    detach();
    scaleX.current = 0;
    scaleY.current = 0;

    if (frame.current) {
      raf.cancel(frame.current);
      frame.current = null;
    }
  }, []);
  var handleEvent = useCallback(function (evt) {
    if (dragging.current && !attached.current) {
      attach();
      updateScrolling(evt);
    }
  }, [dragDropManager]);
  useEffect(function () {
    if (!dragging.current && monitor.isDragging()) {
      dragging.current = true;
    } else if (dragging.current && !monitor.isDragging()) {
      dragging.current = false;
      stopScrolling();
    }
  }, [monitor.isDragging()]);
  useEffect(function () {
    if (!componentRef.current) {
      return function () {};
    }

    componentRef.current.addEventListener('dragover', handleEvent); // touchmove events don't seem to work across siblings, so we unfortunately
    // have to attach the listeners to the body

    window.document.body.addEventListener('touchmove', handleEvent);
    return function () {
      if (componentRef.current) {
        componentRef.current.removeEventListener('dragover', handleEvent);
      }

      window.document.body.removeEventListener('touchmove', handleEvent);
      stopScrolling();
    };
  }, [dragDropManager]);
}
export default function createScrollingComponent(WrappedComponent) {
  function ScrollingComponent(_ref3) {
    var strengthMultiplier = _ref3.strengthMultiplier,
        verticalStrength = _ref3.verticalStrength,
        horizontalStrength = _ref3.horizontalStrength,
        onScrollChange = _ref3.onScrollChange,
        dragDropManager = _ref3.dragDropManager,
        passedProps = _objectWithoutPropertiesLoose(_ref3, _excluded);

    var ref = useRef(null);
    useDndScrolling(ref, {
      strengthMultiplier: strengthMultiplier,
      verticalStrength: verticalStrength,
      horizontalStrength: horizontalStrength,
      onScrollChange: onScrollChange,
      dragDropManager: dragDropManager
    });
    return /*#__PURE__*/_jsx(WrappedComponent, _extends({}, passedProps, {
      ref: ref
    }));
  }

  ScrollingComponent.displayName = "Scrolling(" + getDisplayName(WrappedComponent) + ")";
  ScrollingComponent.propTypes = {
    onScrollChange: PropTypes.func,
    verticalStrength: PropTypes.func,
    horizontalStrength: PropTypes.func,
    strengthMultiplier: PropTypes.number,
    dragDropManager: PropTypes.any
  };
  ScrollingComponent.defaultProps = defaultProps;
  return hoist(ScrollingComponent, WrappedComponent);
}